@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25554 $
+ * $Revision: 27358 $
  * $Author: rjongbloed $
- * $Date: 2011-04-13 03:34:50 -0500 (Wed, 13 Apr 2011) $
+ * $Date: 2012-03-28 21:17:23 -0500 (Wed, 28 Mar 2012) $
  */
 
 
@@ -422,6 +422,22 @@ class PAbstractSet : public PHashTable
     virtual PINDEX GetValuesIndex(
       const PObject & obj   ///< Object to find equal value.
     ) const;
+
+    /**Calculate union of sets.
+       Returns true if any new elements were added.
+      */
+    bool Union(
+      const PAbstractSet & set
+    );
+
+    /**Calculate intersection of sets.
+       Returns true if there is an intersection.
+      */
+    static bool Intersection(
+      const PAbstractSet & set1,
+      const PAbstractSet & set2,
+      PAbstractSet * intersection = NULL
+    );
   //@}
 };
 
