@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26129 $
+ * $Revision: 27397 $
  * $Author: rjongbloed $
- * $Date: 2011-07-07 21:42:50 -0500 (Thu, 07 Jul 2011) $
+ * $Date: 2012-04-01 18:18:04 -0500 (Sun, 01 Apr 2012) $
  */
 
 #ifndef PTLIB_OBJECT_H
@@ -58,6 +58,10 @@
 #include <iomanip>
 #include <iostream>
 #include <sstream>
+#include <vector>
+#include <list>
+#include <map>
+#include <algorithm>
 
 
 #define P_REMOVE_VIRTUAL_INTERNAL_BASE(fn) __inline virtual struct ptlib_virtual_function_changed_or_removed ****** fn { return 0; }
@@ -1051,8 +1055,8 @@ of compatibility with documentation systems.
       { return strcmp(clsName, cls::Class()) == 0 || par::InternalIsDescendant(clsName); } \
     virtual const char * GetClass(unsigned ancestor = 0) const \
       { return ancestor > 0 ? par::GetClass(ancestor-1) : cls::Class(); } \
-    virtual PObject::Comparison CompareObjectMemoryDirect(const PObject & _obj) const \
-      { return (PObject::Comparison)memcmp(this, &_obj, sizeof(cls)); } \
+    virtual PObject::Comparison CompareObjectMemoryDirect(const PObject & obj) const \
+      { return PObject::InternalCompareObjectMemoryDirect(this, dynamic_cast<const cls *>(&obj), sizeof(cls)); } \
     PNEW_AND_DELETE_FUNCTIONS
 
 
@@ -1213,6 +1217,13 @@ class PObject {
       const PObject & obj   // Object to compare against.
     ) const;
 
+    /// Internal function caled from CompareObjectMemoryDirect()
+    static Comparison InternalCompareObjectMemoryDirect(
+      const PObject * obj1,
+      const PObject * obj2,
+      PINDEX size
+    );
+
     /** Compare the two objects.
     
        @return
