@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25554 $
+ * $Revision: 27275 $
  * $Author: rjongbloed $
- * $Date: 2011-04-13 03:34:50 -0500 (Wed, 13 Apr 2011) $
+ * $Date: 2012-03-22 16:51:22 -0500 (Thu, 22 Mar 2012) $
  */
  
 #ifndef PTLIB_SAFE_COLLECTION_H
@@ -579,6 +579,9 @@ class PSafePtrBase : public PObject
     };
     void ExitSafetyMode(ExitSafetyModeOption ref);
 
+    virtual void LockPtr() { }
+    virtual void UnlockPtr() { }
+
   protected:
     const PSafeCollection * collection;
     PSafeObject           * currentObject;
@@ -697,8 +700,8 @@ class PSafePtrMultiThreaded : public PSafePtrBase
     virtual void Previous();
     virtual void DeleteObject(PSafeObject * obj);
 
-    void Lock() { m_mutex.Wait(); }
-    void Unlock();
+    virtual void LockPtr() { m_mutex.Wait(); }
+    virtual void UnlockPtr();
 
   protected:
     mutable PMutex m_mutex;
@@ -795,7 +798,7 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     PSafePtr & operator=(const PSafeCollection & safeCollection)
       {
-        Assign(safeCollection);
+        BaseClass::Assign(safeCollection);
         return *this;
       }
 
@@ -831,24 +834,37 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
      */
     PSafePtr & operator=(PINDEX idx)
       {
-        Assign(idx);
+        BaseClass::Assign(idx);
         return *this;
       }
+
+    /**Set the safe pointer to the specified object.
+       This will return a PSafePtr for the previous value of this. It does
+       this in a thread safe manner so "test and set" semantics are obeyed.
+      */
+    PSafePtr Set(T * obj)
+      {
+        this->LockPtr();
+        PSafePtr oldPtr = *this;
+        this->Assign(obj);
+        this->UnlockPtr();
+        return oldPtr;
+      }
   //@}
 
   /**@name Operations */
   //@{
     /**Return the physical pointer to the object.
       */
-    operator T*()    const { return  (T *)currentObject; }
+    operator T*()    const { return  (T *)BaseClass::currentObject; }
 
     /**Return the physical pointer to the object.
       */
-    T & operator*()  const { return *(T *)PAssertNULL(currentObject); }
+    T & operator*()  const { return *(T *)PAssertNULL(BaseClass::currentObject); }
 
     /**Allow access to the physical object the pointer is pointing to.
       */
-    T * operator->() const { return  (T *)PAssertNULL(currentObject); }
+    T * operator->() const { return  (T *)PAssertNULL(BaseClass::currentObject); }
 
     /**Post-increment the pointer.
        This requires that the pointer has been created with a PSafeCollection
@@ -856,8 +872,8 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator++(int)
       {
-        T * previous = (T *)currentObject;
-        Next();
+        T * previous = (T *)BaseClass::currentObject;
+        BaseClass::Next();
         return previous;
       }
 
@@ -867,8 +883,8 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator++()
       {
-        Next();
-        return (T *)currentObject;
+        BaseClass::Next();
+        return (T *)BaseClass::currentObject;
       }
 
     /**Post-decrement the pointer.
@@ -877,8 +893,8 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator--(int)
       {
-        T * previous = (T *)currentObject;
-        Previous();
+        T * previous = (T *)BaseClass::currentObject;
+        BaseClass::Previous();
         return previous;
       }
 
@@ -888,25 +904,10 @@ template <class T, class BaseClass = PSafePtrBase> class PSafePtr : public BaseC
       */
     T * operator--()
       {
-        Previous();
-        return (T *)currentObject;
+        BaseClass::Previous();
+        return (T *)BaseClass::currentObject;
       }
   //@}
-
-  /**Cast the pointer to a different type. The pointer being cast to MUST
-     be a derived class or NULL is returned.
-    */
-      /*
-  template <class Base>
-  static PSafePtr<T> DownCast(const PSafePtr<Base> & oldPtr)
-  {
-    PSafePtr<T> newPtr;
-    Base * realPtr = oldPtr;
-    if (realPtr != NULL && PIsDescendant(realPtr, T))
-      newPtr.Assign(oldPtr);
-    return newPtr;
-  }
-  */
 };
 
 
